@@ -45,11 +45,17 @@ def standard_config(n_cpus=1):
 
     CONFIG["ignore.invalid_score_columns"] = False
     info["ignore.invalid_score_columns"] =\
-        """ignore score columns which conly contain NaN of infinity values"""
+        """ignore score columns which only contain NaN of infinity values"""
 
     CONFIG["apply"] = None
     info["apply"] = r"""[name of *_scorer.bin file of existing classifier]"""
 
+    CONFIG["export.mayu"] = False
+    info["export.mayu"] = """[export input files for MAYU]"""
+
+    CONFIG["compute.probabilities"] = False
+    info["compute.probabilities"] = """[Compute approximate binned probability values]"""
+
     return CONFIG, info
 
 CONFIG, __ = standard_config()
